@@ -336,7 +336,7 @@ class QState(ctypes.Structure):
         qs = self.__class__(vector=vec)
         return qs
 
-    def show(self, qid=None, nonzero=False):
+    def show(self, qid=None, nonzero=False, preal=0):
         """
         show the quantum state
         (elements of the state vector and probabilities).
@@ -347,6 +347,9 @@ class QState(ctypes.Structure):
             qubit id's list to show.
         nonzero : bool, default False
             if True, only non-zero amplitudes are printed.
+        preal : int, default 0
+            state id to make positive real amplitude.
+            (if -1 is set, do not go out the global phase factor)
 
         Returns
         -------
@@ -379,7 +382,36 @@ class QState(ctypes.Structure):
         c[11] = +0.7071+0.0000*i : 0.5000 |++++++
 
         """
-        qstate_print(self, qid, nonzero)
+        # qstate_print(self, qid, nonzero)
+
+        if qid is None or len(qid) == 0:
+            digits = self.qubit_num
+        else:
+            digits = len(qid)
+
+        vec = self.get_amp(qid=qid)
+
+        if preal >= 0:
+            exp_i_phase = 1.+0.j
+            if abs(vec[preal].imag) > cfg.EPS:
+                exp_i_phase = vec[preal] / abs(vec[preal])
+            elif vec[preal].real < 0.0:
+                exp_i_phase = -exp_i_phase
+            vec = vec / exp_i_phase
+
+        for i, v in enumerate(vec):
+            bits = "{:0{digits}b}".format(i, digits=digits)
+            absval2 = abs(v) * abs(v)
+            if absval2 < cfg.EPS:
+                bar_len = 0
+            else:
+                bar_len = int(absval2 / 0.1 + 1.5)
+            bar_str = "|" + "+" * bar_len
+            if nonzero is True and absval2 < cfg.EPS:
+                continue
+            else:
+                print("c[{}] = {:+.4f}{:+.4f}*i : {:.4f} {}"
+                      .format(bits, v.real, v.imag, abs(v)**2, bar_str))
 
     def clone(self):
         """
