@@ -17,7 +17,7 @@ from qlazy.CMem import CMem
 lib = ctypes.CDLL(str(pathlib.Path(__file__).with_name('libqlz.'+get_lib_ext())))
 libc = ctypes.CDLL(find_library("c"), mode=ctypes.RTLD_GLOBAL)
 
-def qstate_init(qubit_num=None, seed=None):
+def qstate_init(qubit_num=None, seed=None, use_gpu=False):
     """ initialize QState object """
 
     libc.srand(ctypes.c_int(seed))
@@ -25,16 +25,16 @@ def qstate_init(qubit_num=None, seed=None):
     qstate = None
     c_qstate = ctypes.c_void_p(qstate)
 
-    lib.qstate_init.restype = ctypes.c_int
-    lib.qstate_init.argtypes = [ctypes.c_int, ctypes.POINTER(ctypes.c_void_p)]
-    ret = lib.qstate_init(ctypes.c_int(qubit_num), c_qstate)
+    lib.qstate_init.restype = ctypes.c_bool
+    lib.qstate_init.argtypes = [ctypes.c_int, ctypes.POINTER(ctypes.c_void_p), ctypes.c_bool]
+    ret = lib.qstate_init(ctypes.c_int(qubit_num), c_qstate, ctypes.c_bool(use_gpu))
 
-    if ret == cfg.FALSE:
+    if ret is False:
         raise ValueError("can't initialize QState object.")
 
     return c_qstate
 
-def qstate_init_with_vector(vector=None, seed=None):
+def qstate_init_with_vector(vector=None, seed=None, use_gpu=False):
     """ initialize QState object with vector """
 
     libc.srand(ctypes.c_int(seed))
@@ -53,13 +53,13 @@ def qstate_init_with_vector(vector=None, seed=None):
     c_vec_real = DoubleArray(*vec_real)
     c_vec_imag = DoubleArray(*vec_imag)
 
-    lib.qstate_init_with_vector.restype = ctypes.c_int
+    lib.qstate_init_with_vector.restype = ctypes.c_bool
     lib.qstate_init_with_vector.argtypes = [DoubleArray, DoubleArray, ctypes.c_int,
-                                            ctypes.POINTER(ctypes.c_void_p)]
+                                            ctypes.POINTER(ctypes.c_void_p), ctypes.c_bool]
     ret = lib.qstate_init_with_vector(c_vec_real, c_vec_imag, ctypes.c_int(dim),
-                                      c_qstate)
+                                      c_qstate, ctypes.c_bool(use_gpu))
 
-    if ret == cfg.FALSE:
+    if ret is False:
         raise ValueError("can't initialize QState object.")
 
     return c_qstate
@@ -82,11 +82,11 @@ def qstate_reset(qs, qid=None):
         IntArray = ctypes.c_int * qubit_num
         qid_array = IntArray(*qubit_id)
 
-        lib.qstate_reset.restype = ctypes.c_int
+        lib.qstate_reset.restype = ctypes.c_bool
         lib.qstate_reset.argtypes = [ctypes.POINTER(QState), ctypes.c_int, IntArray]
         ret = lib.qstate_reset(ctypes.byref(qs), ctypes.c_int(qubit_num), qid_array)
 
-        if ret == cfg.FALSE:
+        if ret is False:
             raise ValueError("can't reset quantum state vector.")
 
     except Exception:
@@ -110,12 +110,12 @@ def qstate_print(qs, qid=None, nonzero=False):
         IntArray = ctypes.c_int * qubit_num
         qid_array = IntArray(*qubit_id)
 
-        lib.qstate_print.restype = ctypes.c_int
+        lib.qstate_print.restype = ctypes.c_bool
         lib.qstate_print.argtypes = [ctypes.POINTER(QState), ctypes.c_int, IntArray, ctypes.c_bool]
         ret = lib.qstate_print(ctypes.byref(qs), ctypes.c_int(qubit_num),
                                qid_array, ctypes.c_bool(nonzero))
 
-        if ret == cfg.FALSE:
+        if ret is False:
             raise ValueError("can't print quantum state vector.")
 
     except Exception:
@@ -129,12 +129,12 @@ def qstate_copy(qs):
         qstate = None
         c_qstate = ctypes.c_void_p(qstate)
 
-        lib.qstate_copy.restype = ctypes.c_int
+        lib.qstate_copy.restype = ctypes.c_bool
         lib.qstate_copy.argtypes = [ctypes.POINTER(QState),
                                     ctypes.POINTER(ctypes.c_void_p)]
         ret = lib.qstate_copy(ctypes.byref(qs), c_qstate)
 
-        if ret == cfg.FALSE:
+        if ret is False:
             raise ValueError("can't copy quantum state vector.")
 
         return c_qstate
@@ -155,14 +155,14 @@ def qstate_bloch(qs, q=0):
         c_theta = ctypes.c_double(theta)
         c_phi = ctypes.c_double(phi)
 
-        lib.qstate_bloch.restype = ctypes.c_int
+        lib.qstate_bloch.restype = ctypes.c_bool
         lib.qstate_bloch.argtypes = [ctypes.POINTER(QState), ctypes.c_int,
                                      ctypes.POINTER(ctypes.c_double),
                                      ctypes.POINTER(ctypes.c_double)]
         ret = lib.qstate_bloch(ctypes.byref(qs), ctypes.c_int(q),
                                ctypes.byref(c_theta), ctypes.byref(c_phi))
 
-        if ret == cfg.FALSE:
+        if ret is False:
             raise ValueError("can't get bloch angle.")
 
         theta = c_theta.value
@@ -183,7 +183,7 @@ def qstate_inner_product(qs_0, qs_1):
         c_real = ctypes.c_double(real)
         c_imag = ctypes.c_double(imag)
 
-        lib.qstate_inner_product.restype = ctypes.c_int
+        lib.qstate_inner_product.restype = ctypes.c_bool
         lib.qstate_inner_product.argtypes = [ctypes.POINTER(QState),
                                              ctypes.POINTER(QState),
                                              ctypes.POINTER(ctypes.c_double),
@@ -191,7 +191,7 @@ def qstate_inner_product(qs_0, qs_1):
         ret = lib.qstate_inner_product(ctypes.byref(qs_0), ctypes.byref(qs_1),
                                        ctypes.byref(c_real), ctypes.byref(c_imag))
 
-        if ret == cfg.FALSE:
+        if ret is False:
             raise ValueError("can't get inner product of 2 quantum state vectors.")
 
         real = c_real.value
@@ -228,13 +228,13 @@ def qstate_get_camp(qs, qid=None):
 
         camp = None
         c_camp = ctypes.c_void_p(camp)
-        lib.qstate_get_camp.restype = ctypes.c_int
+        lib.qstate_get_camp.restype = ctypes.c_bool
         lib.qstate_get_camp.argtypes = [ctypes.POINTER(QState), ctypes.c_int, IntArray,
                                         ctypes.POINTER(ctypes.c_void_p)]
         ret = lib.qstate_get_camp(ctypes.byref(qs), ctypes.c_int(qubit_num),
                                   qid_array, c_camp)
 
-        if ret == cfg.FALSE:
+        if ret is False:
             raise ValueError("can't get element of the quantum state vector.")
 
         o = ctypes.cast(c_camp.value, ctypes.POINTER(ctypes.c_double))
@@ -254,20 +254,20 @@ def qstate_get_camp(qs, qid=None):
 
 
 def qstate_tensor_product(qs, qstate):
-    """ get tensor procuct of 2 quantum state vectors """
+    """ get tensor product of 2 quantum state vectors """
 
     try:
         qstate_out = None
         c_qstate_out = ctypes.c_void_p(qstate_out)
 
-        lib.qstate_tensor_product.restype = ctypes.c_int
+        lib.qstate_tensor_product.restype = ctypes.c_bool
         lib.qstate_tensor_product.argtypes = [ctypes.POINTER(QState),
                                               ctypes.POINTER(QState),
                                               ctypes.POINTER(ctypes.c_void_p)]
         ret = lib.qstate_tensor_product(ctypes.byref(qs), ctypes.byref(qstate),
                                         c_qstate_out)
 
-        if ret == cfg.FALSE:
+        if ret is False:
             raise ValueError("can't get tensor product of the 2 quantum state vectors.")
 
         return c_qstate_out
@@ -286,13 +286,13 @@ def qstate_evolve(qs, observable=None, time=0.0, iteration=0):
         raise ValueError("observable must be set.")
 
     try:
-        lib.qstate_evolve.restype = ctypes.c_int
+        lib.qstate_evolve.restype = ctypes.c_bool
         lib.qstate_evolve.argtypes = [ctypes.POINTER(QState), ctypes.POINTER(Observable),
                                       ctypes.c_double, ctypes.c_int]
         ret = lib.qstate_evolve(ctypes.byref(qs), ctypes.byref(observable),
                                 ctypes.c_double(time), ctypes.c_int(iteration))
 
-        if ret == cfg.FALSE:
+        if ret is False:
             raise ValueError("can't get the quantum state vectors after time evolution.")
 
     except Exception:
@@ -309,7 +309,7 @@ def qstate_expect_value(qs, observable=None):
     try:
         val = 0.0
         c_val = ctypes.c_double(val)
-        lib.qstate_expect_value.restype = ctypes.c_int
+        lib.qstate_expect_value.restype = ctypes.c_bool
         lib.qstate_expect_value.argtypes = [ctypes.POINTER(QState),
                                             ctypes.POINTER(Observable),
                                             ctypes.POINTER(ctypes.c_double)]
@@ -317,7 +317,7 @@ def qstate_expect_value(qs, observable=None):
                                       ctypes.byref(observable),
                                       ctypes.byref(c_val))
 
-        if ret == cfg.FALSE:
+        if ret is False:
             raise ValueError("can't get expect value of the observable"
                              " under the quantum state vector.")
 
@@ -368,7 +368,7 @@ def qstate_apply_matrix(qs, matrix=None, qid=None):
         c_mat_real = DoubleArray(*mat_real)
         c_mat_imag = DoubleArray(*mat_imag)
 
-        lib.qstate_apply_matrix.restype = ctypes.c_int
+        lib.qstate_apply_matrix.restype = ctypes.c_bool
         lib.qstate_apply_matrix.argtypes = [ctypes.POINTER(QState),
                                             ctypes.c_int, IntArray,
                                             DoubleArray, DoubleArray,
@@ -378,7 +378,7 @@ def qstate_apply_matrix(qs, matrix=None, qid=None):
                                       c_mat_real, c_mat_imag,
                                       ctypes.c_int(row), ctypes.c_int(col))
 
-        if ret == cfg.FALSE:
+        if ret is False:
             raise ValueError("can't apply the matrix to the quantum state vector.")
 
     except Exception:
@@ -398,7 +398,7 @@ def qstate_operate_qgate(qs, kind=None, qid=None, phase=cfg.DEF_PHASE,
     IntArray = ctypes.c_int * 2
     qid_array = IntArray(*qubit_id)
 
-    lib.qstate_operate_qgate.restype = ctypes.c_int
+    lib.qstate_operate_qgate.restype = ctypes.c_bool
     lib.qstate_operate_qgate.argtypes = [ctypes.POINTER(QState), ctypes.c_int,
                                          ctypes.c_double, ctypes.c_double,
                                          ctypes.c_double, IntArray]
@@ -406,42 +406,48 @@ def qstate_operate_qgate(qs, kind=None, qid=None, phase=cfg.DEF_PHASE,
                                    ctypes.c_double(phase), ctypes.c_double(phase1),
                                    ctypes.c_double(phase2), qid_array)
 
-    if ret == cfg.FALSE:
+    if ret is False:
         raise ValueError("can't operate quantum gate to the quantum state vector.")
 
-def qstate_measure(qs, qid=None, angle=0.0, phase=0.0):
+def qstate_measure(qs, qid=None):
     """ measurement of the qubits """
 
+    # qnum, mnum
+    qnum = qs.qubit_num
     if qid is None or qid == []:
-        qid = list(range(qs.qubit_num))
+        qid = list(range(qnum))
+    mnum = len(qid)
 
-    # operate
-    qubit_num = len(qid)
-    qubit_id = [0 for _ in range(qubit_num)]
+    # qid_array
+    qubit_id = [0 for _ in range(qnum)]
     for i, q in enumerate(qid):
         qubit_id[i] = q
-    IntArray = ctypes.c_int * qubit_num
+    IntArray = ctypes.c_int * qnum
     qid_array = IntArray(*qubit_id)
 
-    mval = 0
-    c_mval = ctypes.c_int(mval)
+    # mstr_all_array, mstr_qid_array
+    mstr = "0" * qnum
+    mstr_array = ctypes.create_string_buffer(mstr.encode())
 
-    lib.qstate_measure.restype = ctypes.c_int
-    lib.qstate_measure.argtypes = [ctypes.POINTER(QState),
-                                   ctypes.c_double, ctypes.c_double,
-                                   ctypes.c_int, IntArray, ctypes.POINTER(ctypes.c_int)]
-    ret = lib.qstate_measure(ctypes.byref(qs),
-                             ctypes.c_double(angle), ctypes.c_double(phase),
-                             ctypes.c_int(qubit_num), qid_array, ctypes.byref(c_mval))
 
-    if ret == cfg.FALSE:
+
+    mchar_bytes = bytearray([0x00 for _ in range(qnum)])
+    CharArray = ctypes.c_char * qnum
+    mchar_array = CharArray(*mchar_bytes)
+    
+    lib.qstate_measure.restype = ctypes.c_bool
+    lib.qstate_measure.argtypes = [ctypes.POINTER(QState), ctypes.c_int,
+                                   IntArray, ctypes.c_char_p, ctypes.c_bool]
+    ret = lib.qstate_measure(ctypes.byref(qs), ctypes.c_int(mnum),
+                             qid_array, mchar_array, True)
+
+    if ret is False:
         raise ValueError("can't measure the qubits.")
 
-    mval = c_mval.value
-    digits = len(qid)
-    mval = '{:0{digits}b}'.format(mval, digits=digits)
+    measured_str = mstr_array.value.decode()
+    measured_str = "".join(map(str, [int.from_bytes(c, byteorder='big') for c in mchar_array]))
 
-    return mval
+    return measured_str
 
 def qstate_measure_stats(qs, qid=None, shots=cfg.DEF_SHOTS, angle=0.0, phase=0.0):
     """ measurement of the qubits and get stats """
@@ -463,7 +469,7 @@ def qstate_measure_stats(qs, qid=None, shots=cfg.DEF_SHOTS, angle=0.0, phase=0.0
     mdata = None
     c_mdata = ctypes.c_void_p(mdata)
 
-    lib.qstate_measure_stats.restype = ctypes.c_int
+    lib.qstate_measure_stats.restype = ctypes.c_bool
     lib.qstate_measure_stats.argtypes = [ctypes.POINTER(QState), ctypes.c_int,
                                          ctypes.c_double, ctypes.c_double,
                                          ctypes.c_int, IntArray,
@@ -472,7 +478,7 @@ def qstate_measure_stats(qs, qid=None, shots=cfg.DEF_SHOTS, angle=0.0, phase=0.0
                                    ctypes.c_double(angle), ctypes.c_double(phase),
                                    ctypes.c_int(qubit_num), qid_array, c_mdata)
 
-    if ret == cfg.FALSE:
+    if ret is False:
         raise ValueError("can't measure the qubits.")
 
     out = ctypes.cast(c_mdata.value, ctypes.POINTER(MData))
@@ -499,63 +505,57 @@ def qstate_measure_bell_stats(qs, qid=None, shots=cfg.DEF_SHOTS):
     mdata = None
     c_mdata = ctypes.c_void_p(mdata)
 
-    lib.qstate_measure_bell_stats.restype = ctypes.c_int
+    lib.qstate_measure_bell_stats.restype = ctypes.c_bool
     lib.qstate_measure_bell_stats.argtypes = [ctypes.POINTER(QState), ctypes.c_int,
                                               ctypes.c_int, IntArray,
                                               ctypes.POINTER(ctypes.c_void_p)]
-
     ret = lib.qstate_measure_bell_stats(ctypes.byref(qs), ctypes.c_int(shots),
                                         ctypes.c_int(qubit_num), qid_array, c_mdata)
 
-    if ret == cfg.FALSE:
+    if ret is False:
         raise ValueError("can't measure the qubits.")
 
     out = ctypes.cast(c_mdata.value, ctypes.POINTER(MData))
 
     return out.contents
 
-def qstate_operate_qcirc(qstate, cmem, qcirc, shots, cid):
+def qstate_operate_qcirc(qstate, cmem, qcirc, shots, cid, out_state):
     """ operate quantum circuit """
 
-    lib.qstate_operate_qcirc.restype = ctypes.c_int
+    if cmem is not None:
+        cmem_num = cmem.cmem_num
+    else:
+        cmem_num = 0
+
+    buf_size = cmem_num * shots
+    mchar_array = bytearray([0x00 for _ in range(buf_size)])
+    CharArray = ctypes.c_char * buf_size
+    mchar_shots = CharArray(*mchar_array)
+
+    lib.qstate_operate_qcirc.restype = ctypes.c_bool
     lib.qstate_operate_qcirc.argtypes = [ctypes.POINTER(QState),
-                                         ctypes.POINTER(CMem), ctypes.POINTER(QCirc)]
+                                         ctypes.POINTER(CMem), ctypes.POINTER(QCirc),
+                                         ctypes.c_int, CharArray, ctypes.c_bool]
 
     if cmem is not None:
-
-        cmem_num = cmem.cmem_num
+        ret = lib.qstate_operate_qcirc(ctypes.byref(qstate),
+                                       ctypes.byref(cmem), ctypes.byref(qcirc),
+                                       ctypes.c_int(shots), mchar_shots, ctypes.c_bool(out_state))
         frequency = Counter()
-        for n in range(shots):
-
-            if n < shots - 1:
-                qstate_tmp = qstate.clone()
-                cmem_tmp = cmem.clone()
-                ret = lib.qstate_operate_qcirc(ctypes.byref(qstate_tmp),
-                                               ctypes.byref(cmem_tmp), ctypes.byref(qcirc))
-                bit_array = ctypes.cast(cmem_tmp.bit_array, ctypes.POINTER(ctypes.c_ubyte*cmem_num))
-            else:
-                ret = lib.qstate_operate_qcirc(ctypes.byref(qstate), ctypes.byref(cmem),
-                                               ctypes.byref(qcirc))
-                bit_array = ctypes.cast(cmem.bit_array, ctypes.POINTER(ctypes.c_ubyte*cmem_num))
-
-            if ret == cfg.FALSE:
-                raise ValueError("can't operate the quantum circuit.")
-
-            cmem_list = [bit_array.contents[i] for i in range(cmem_num)]
+        for i in range(0, len(mchar_shots), cmem_num):
+            cmem_list = mchar_shots[i:i+cmem_num]
             cmem_list_part = [cmem_list[c] for c in cid]
-            mval = "".join(map(str, cmem_list_part))
+            mchar = "".join(map(str, cmem_list_part))
+            frequency[mchar] += 1
 
-            frequency[mval] += 1
+    else: # unitary only
+        c_cmem = ctypes.POINTER(CMem)()
+        ret = lib.qstate_operate_qcirc(ctypes.byref(qstate), c_cmem, ctypes.byref(qcirc),
+                                       ctypes.c_int(shots), mchar_shots, ctypes.c_bool(out_state))
 
-        return frequency
+        frequency = None
 
-    c_cmem = ctypes.POINTER(CMem)()
-    ret = lib.qstate_operate_qcirc(ctypes.byref(qstate), c_cmem, ctypes.byref(qcirc))
-
-    if ret == cfg.FALSE:
-        raise ValueError("can't operate the quantum circuit.")
-
-    return None
+    return frequency
 
 def qstate_free(qs):
     """ free memory of the QState object """
